#include <iostream>
#include <vector>

using namespace std;

struct Video {
    string name;
    int length;
};

struct User {
    int id;
    string name;
    string surname;
    int subDuration;
    bool avaliable;
};

class VoD {
    vector<User> users;
    vector<Video> videos;

public:
    void addUser(string name, string surname, int subDuration) {
        int len = users.size();
        User tmp;
        tmp.id = len;
        tmp.name = name;
        tmp.surname = surname;
        tmp.subDuration = subDuration;
        tmp.avaliable = true;
        users.push_back(tmp);
    }

    void removeUser(int id) {
        /*users.erase(
            remove_if(users.begin(), users.end(), [&](User const& user) {
                return user.id == id;
                }),
            users.end());*/
        users.at(id).avaliable = false;
    }

    void extendSub(int userId, int days) {
        if (!users.at(userId).avaliable) cout << "User with ID " << userId << " is not avaliable" << endl;
        users.at(userId).subDuration = users.at(userId).subDuration + days;
    }

    void watchVideo(int userId) {
        if (!users.at(userId).avaliable) cout << "User with ID " << userId << " is not avaliable" << endl;
        else if (users.at(userId).subDuration > 0) cout << "User " << users.at(userId).name << " can watch" << endl;
        else cout << "User " << users.at(userId).name << " cannot watch (expired " << (users.at(userId).subDuration * -1) << " days ago)" << endl;
    }

    friend void displayUsers(VoD);
    friend void displayVideos(VoD);

    bool isSubValidForUser(int id) {
        User user = users.at(id);
        if (user.subDuration > 0 && users.at(id).avaliable) return true;
        return false;
    }

    void addVideo(string name, int length) {
        Video tmp;
        tmp.name = name;
        tmp.length = length;
        videos.push_back(tmp);
    }

    void removeVideo(string name) {
        videos.erase(
            remove_if(videos.begin(), videos.end(), [&](Video const& video) {
                return video.name == name;
                }),
            videos.end());
    }
};

void displayUsers(VoD vod) {
    for (vector<User>::iterator i = vod.users.begin(); i != vod.users.end(); i++) {
        User user = *i;
        if (!user.avaliable) continue; // iterowanie wektora z "pustym" polem wywalao segfault, nie wiem dlaczego skoro podejcie z displayVideos() dziaa tam prawidowo
        cout << "ID: " << user.id << ", " << user.name << " " << user.surname;
        if (vod.isSubValidForUser(user.id)) cout << ", sub valid for " << user.subDuration << " days" << endl;
        else cout << ", sub expired " << (user.subDuration * -1) << " days ago" << endl; // ujemne wartoci subDuration to wygasy abonament, mnoymy przez -1 by zamieni na normalne dni
    }
}

void displayVideos(VoD vod) {
    for (vector<Video>::iterator i = vod.videos.begin(); i != vod.videos.end(); i++) {
        Video vid = *i;
        cout << "Name: " << vid.name << ", length: " << vid.length << " minutes" << endl;
    }
}

int main()
{
    VoD vod;
    vod.addVideo("Interstellar", 180);
    vod.addVideo("The Martian", 125);
    vod.addVideo("Gravity", 100);
    displayVideos(vod);
    vod.removeVideo("The Martian");
    displayVideos(vod);
    vod.addUser("Andrzej", "Kretoszczur", 7);
    vod.addUser("Stefan", "Myszojelen", 14);
    vod.addUser("Jakub", "Dodo", 21);
    displayUsers(vod);
    vod.removeUser(1);
    displayUsers(vod);
    vod.extendSub(0, -35);
    vod.extendSub(1, 14); 
    vod.extendSub(2, 7);
    displayUsers(vod);
    vod.watchVideo(0);
    vod.watchVideo(2);
}